@@ -5,12 +5,12 @@ from datetime import date, datetime
 import pandas as pd
 import numpy as np
 import logging
-
-LOG_FORMAT = "%(asctime)s:%(levelname)s:%(name)s:%(funcName)s:%(message)s"
+import Longger_design
+# LOG_FORMAT = "%(asctime)s:%(levelname)s:%(name)s:%(funcName)s:%(message)s"
+# # logging.basicConfig(filename='./CHGraph.log', level=logging.INFO, format=LOG_FORMAT)
 # logging.basicConfig(filename='./CHGraph.log', level=logging.INFO, format=LOG_FORMAT)
-logging.basicConfig(filename='./CHGraph.log', level=logging.WARNING, format=LOG_FORMAT)
-logger = logging.getLogger('CHGraph')
-
+# logger = logging.getLogger('CHGraph')
+logger = Longger_design.get_logger()
 
 def tostr(obj):
     if type(obj) == list:
@@ -348,7 +348,54 @@ class CHGraph(object):
         else:
             return multi_res
 
-    # multi_hop_multi_edge_common_vertexes return common vertexes
+        # 起点列表多跳多边，循环的方式拿到多次一跳多边的结果
+        ## multi_hop_multi_edge return edges
+        ## multi_hop_multi_edge requires type(src) == type(dst)
+
+    def find_path_response(self,
+                           start_vertex_list,
+                           direction,
+                           edge_name_list,
+                           edge_con_list_list,
+                           target_field_list):
+
+        multi_step_start_vertex_list = start_vertex_list
+        res = self.one_hop_multi_edge(multi_step_start_vertex_list,
+                                      direction,
+                                      edge_name_list,
+                                      edge_con_list_list,
+                                      target_field_list)
+
+        # TODO: may have performance issue
+        tmp_list = []
+        if direction == "forward":
+
+            for ii in range(len(edge_name_list)):
+                if res[ii].shape[0] == 0:
+                    tmp_list.append([])
+                    continue
+                edge_name = edge_name_list[ii]
+                edge_info = self.graph_cfg["edges"][edge_name]
+                tmp_list.append(np.unique(res[ii][edge_info['dst']].values))
+            # multi_step_start_vertex_list = np.unique(np.concatenate(tmp_list))
+        elif direction == "backward":
+            for ii in range(len(edge_name_list)):
+                if res[ii].shape[0] == 0:
+                    tmp_list.append([])
+                    continue
+                edge_name = edge_name_list[ii]
+                edge_info = self.graph_cfg["edges"][edge_name]
+                tmp_list.append(np.unique(res[ii][edge_info['src']].values))
+            # multi_step_start_vertex_list = np.unique(np.concatenate(tmp_list))
+        elif direction == "bidirectional":
+            logger.warning("bidirectional not implemented")
+            return "bidirectional not implemented"
+        else:
+            logger.warning("invalid direction")
+            return "invalid direction"
+        return (res, tmp_list)
+
+            # multi_hop_multi_edge_common_vertexes return common vertexes
     def multi_hop_multi_edge_common_vertexes(self,
                                              step,
                                              start_vertex_list,
@@ -694,6 +741,8 @@ class CHGraph(object):
         # subgraph_name is recommended to be unique (ex. plus timestamp)
 
     def update_subgraph_by_multi_hop_multi_edge(self,
+                                                main_graph,
+                                                db,
                                                 subgraph_name,
                                                 step,
                                                 start_vertex_list,
@@ -703,7 +752,7 @@ class CHGraph(object):
 
         multi_res = self.multi_hop_multi_edge(step, start_vertex_list, direction,
                                               edge_name_list, edge_con_list_list, None, False, False)
-
+        self.use_graph(subgraph_name, db)
         try:
             for single_res in multi_res:
                 for i in range(len(edge_name_list)):
@@ -724,22 +773,22 @@ class CHGraph(object):
 
                     src_info = self.graph_cfg["vertexes"][edge_info["src_type"]]
                     src_db_table_new = subgraph_name + "." + src_info["table"]
-                    src_db_table_old = src_info["db"] + "." + src_info["table"]
+                    src_db_table_old = main_graph + "." + src_info["table"]
                     src_target_field_list_con = src_info["id"] + "," + ",".join(src_info["fields"])
                     self.client.execute(
-                        "insert into " + src_db_table_new + " (" + src_target_field_list_con + ") select " + src_target_field_list_con + " from " + src_db_table_old + " where " +
+                        "insert into " + src_db_table_new + " (" + src_target_field_list_con + ") select distinct " + src_target_field_list_con + " from " + src_db_table_old + " where " +
                         src_info["id"] + " in " + tostr(col_list[0]))
                     dst_info = self.graph_cfg["vertexes"][edge_info["dst_type"]]
                     dst_db_table_new = subgraph_name + "." + dst_info["table"]
-                    dst_db_table_old = dst_info["db"] + "." + dst_info["table"]
+                    dst_db_table_old = main_graph + "." + dst_info["table"]
                     dst_target_field_list_con = dst_info["id"] + "," + ",".join(dst_info["fields"])
                     self.client.execute(
-                        "insert into " + dst_db_table_new + " (" + dst_target_field_list_con + ") select " + dst_target_field_list_con + " from " + dst_db_table_old + " where " +
+                        "insert into " + dst_db_table_new + " (" + dst_target_field_list_con + ") select distinct " + dst_target_field_list_con + " from " + dst_db_table_old + " where " +
                         dst_info["id"] + " in " + tostr(col_list[1]))
-            return "The subgraph is update success"
+            return multi_res
         except Exception as e:
             print(e)
-            return "The subgraph is update failed"
+            logger.error(e)
 
     def update_subgraph_by_match_edge(self,
                                       subgraph_name,
@@ -868,6 +917,8 @@ class CHGraph(object):
             logger.error("The subgraph update failed, the type is " + type)
 
     def update_subgraph_by_find_path_multi_edge(self,
+                                                db,
+                                                main_graph,
                                                 subgraph_name,
                                                 start_vertex_list,
                                                 end_vertex_list,
@@ -879,6 +930,7 @@ class CHGraph(object):
                                         edge_name_list, edge_con_list_list,
                                         None, step_limit)
 
+        self.use_graph(subgraph_name, db)
         for i in range(len(res)):
             if i == 0:  # path of length 1
                 for k in range(len(edge_name_list)):
@@ -898,17 +950,17 @@ class CHGraph(object):
                                         columnar=True)
                     src_info = self.graph_cfg["vertexes"][edge_info["src_type"]]
                     src_db_table_new = subgraph_name + "." + src_info["table"]
-                    src_db_table_old = src_info["db"] + "." + src_info["table"]
+                    src_db_table_old = main_graph + "." + src_info["table"]
                     src_target_field_list_con = src_info["id"] + "," + ",".join(src_info["fields"])
                     self.client.execute(
-                        "insert into " + src_db_table_new + " (" + src_target_field_list_con + ") select " + src_target_field_list_con + " from " + src_db_table_old + " where " +
+                        "insert into " + src_db_table_new + " (" + src_target_field_list_con + ") select distinct " + src_target_field_list_con + " from " + src_db_table_old + " where " +
                         src_info["id"] + " in " + tostr(col_list[0]))
                     dst_info = self.graph_cfg["vertexes"][edge_info["dst_type"]]
                     dst_db_table_new = subgraph_name + "." + dst_info["table"]
-                    dst_db_table_old = dst_info["db"] + "." + dst_info["table"]
+                    dst_db_table_old = main_graph + "." + dst_info["table"]
                     dst_target_field_list_con = dst_info["id"] + "," + ",".join(dst_info["fields"])
                     self.client.execute(
-                        "insert into " + dst_db_table_new + " (" + dst_target_field_list_con + ") select " + dst_target_field_list_con + " from " + dst_db_table_old + " where " +
+                        "insert into " + dst_db_table_new + " (" + dst_target_field_list_con + ") select distinct " + dst_target_field_list_con + " from " + dst_db_table_old + " where " +
                         dst_info["id"] + " in " + tostr(col_list[1]))
 
             else:  # path of length >=2
@@ -930,18 +982,19 @@ class CHGraph(object):
                                             col_list, columnar=True)
                         src_info = self.graph_cfg["vertexes"][edge_info["src_type"]]
                         src_db_table_new = subgraph_name + "." + src_info["table"]
-                        src_db_table_old = src_info["db"] + "." + src_info["table"]
+                        src_db_table_old = main_graph + "." + src_info["table"]
                         src_target_field_list_con = src_info["id"] + "," + ",".join(src_info["fields"])
                         self.client.execute(
                             "insert into " + src_db_table_new + " (" + src_target_field_list_con + ") select " + src_target_field_list_con + " from " + src_db_table_old + " where " +
                             src_info["id"] + " in " + tostr(col_list[0]))
                         dst_info = self.graph_cfg["vertexes"][edge_info["dst_type"]]
                         dst_db_table_new = subgraph_name + "." + dst_info["table"]
-                        dst_db_table_old = dst_info["db"] + "." + dst_info["table"]
+                        dst_db_table_old = main_graph + "." + dst_info["table"]
                         dst_target_field_list_con = dst_info["id"] + "," + ",".join(dst_info["fields"])
                         self.client.execute(
                             "insert into " + dst_db_table_new + " (" + dst_target_field_list_con + ") select " + dst_target_field_list_con + " from " + dst_db_table_old + " where " +
                             dst_info["id"] + " in " + tostr(col_list[1]))
+        return res
 
     #############################################
     #################### vertex/edge-wise operation: metric
@@ -1092,7 +1145,7 @@ class CHGraph(object):
             edge_con_list_con += " and " + " and ".join(edge_con_list)
 
         num = self.client.execute(
-            "select count(1) from " + db_table + edge_con_list_con)
+            "select count(1) from ( select  distinct " + table_property["src"] + "," + table_property["dst"] + "," + table_property["rank"] + " from " + db_table + edge_con_list_con + " ) " )
         count = num[0][0]
         if page is not None and page_size is not None:
             edge_con_list_con = edge_con_list_con + " limit " + str(page_size * (page - 1)) + "," + str(page_size)
