@@ -3,6 +3,21 @@ from graph_op import CHGraph
 from clickhouse_driver import Client
 from db_op import DBoperator
 import json
+import model_sevice as mds
+import data_load_config
+import numpy as np
+import traceback
+import logging
+import Longger_design
+
+# LOG_FORMAT = "%(asctime)s:%(levelname)s:%(name)s:%(funcName)s:%(message)s"
+# logging.basicConfig(filename='./CHGraph.log', level=logging.INFO, format=LOG_FORMAT)
+# logger = logging.getLogger()
+logger = Longger_design.get_logger()
+
+def get_client(clickhouse_connect):
+    graph_client = Client(host=clickhouse_connect["ip"])
+    return graph_client
 
 
 def is_json(json_string):
@@ -38,21 +53,33 @@ def model_service_ga_build(data, config_params):
         subGraph = data["subGraph"]
     else:
         subGraph = None
-
-    graph = config_params["graph"]
     db = config_params["db"]
+    clickhouse_connect = config_params["clickhouse_connect"]
+    graph_client = get_client(clickhouse_connect)
+    graph = CHGraph(graph_client)
+    # graph = config_params["graph"]
+    # db = config_params["db"]
     graph.use_graph(graphName, db)
 
     graph_cfg = db.use_tables(graphName)
-
+    #开始时间
+    start = datetime.datetime.now()
     db.insert_tables(subGraph, json.dumps(graph_cfg), "sub", graphName)
+    #结束时间
+    end = datetime.datetime.now()
+    logger.info("insert_tables count_times:" + str((end - start).seconds*1000))
     graph.create_subgraph(subGraph)
-
     dict = graph_cfg[type][attrType]
     graph.use_graph(subGraph, db)
-    graph.update_subgraph_by_sql(subGraph, dict, sql, type, attrType, graphName)\
-
+    # 开始时间
+    start = datetime.datetime.now()
+    graph.update_subgraph_by_sql(subGraph, dict, sql, type, attrType, graphName)
+    # 结束时间
+    end = datetime.datetime.now()
+    logger.info("update_subgraph_by_sql count_times:" + str((end - start).seconds*1000))
     graphNodes = []
+    # 开始时间
+    start = datetime.datetime.now()
     for vertex in graph_cfg["vertexes"]:
         graphNode = {}
         dict1 = graph_cfg["vertexes"][vertex]
@@ -70,13 +97,18 @@ def model_service_ga_build(data, config_params):
             )
         except Exception as e:
             print(e)
+            logger.error(e)
             return "vertex query failed"
         graphNode["type"] = vertex
         graphNode["data"] = vertex_query
 
         graphNodes.append(graphNode)
-
+    # 结束时间
+    end = datetime.datetime.now()
+    logger.info("graphNodes.append count_times:" + str((end - start).seconds*1000))
     graphEdges = []
+    # 开始时间
+    start = datetime.datetime.now()
     for edge in graph_cfg["edges"]:
         graphEdge = {}
         dict2 = graph_cfg["edges"][edge]
@@ -95,13 +127,16 @@ def model_service_ga_build(data, config_params):
             )
         except Exception as e:
             print(e)
+            logger.error(e)
             return "edge query failed"
         graphEdge["type"] = edge
         graphEdge["data"] = edges_result
         graphEdge["id"] = edge
 
         graphEdges.append(graphEdge)
-
+    # 结束时间
+    end = datetime.datetime.now()
+    logger.info("graphEdges.append count_times:" + str((end - start).seconds*1000))
     GraphSet = {}
     GraphSet["pathList"] = {}
     GraphSet["columns"] = []
@@ -190,6 +225,7 @@ def model_service_graph_update(data, config_params):
 
     return res
 
+
 def model_service_graph_search_multi_hop_multi_edge(data, config_params):
     '''
 
@@ -1127,20 +1163,24 @@ def model_service_update_subgraph_by_multi_hop_multi_edge(data, config_params):
     if "graph_name" in data.keys():
         graph_name = data["graph_name"]
     else:
-        graph_name = "taobao"
+        # graph_name = "taobao"
+        # return None
+        pass
 
     # client = Client('10.217.62.41')
     #
     # graph = CHGraph(client)
-    graph = config_params["graph"]
+    # graph = config_params["graph"]
 
     db = config_params["db"]
-    graph.use_graph(graph_name, db)
-
+    clickhouse_connect = config_params["clickhouse_connect"]
+    graph_client = get_client(clickhouse_connect)
+    graph = CHGraph(graph_client)
     if "subgraph_name" in data.keys():
         subgraph_name = data["subgraph_name"]
     else:
-        subgraph_name = "taobao_sub"
+        # subgraph_name = "taobao_sub"
+        return None
 
     if "start_vertex_list" in data.keys():
         start_vertex_list = data["start_vertex_list"]
@@ -1155,7 +1195,12 @@ def model_service_update_subgraph_by_multi_hop_multi_edge(data, config_params):
     if "edge_name_list" in data.keys():
         edge_name_list = data["edge_name_list"]
     else:
-        edge_name_list = ["tcpflow", "flow"]
+        return None
+
+    if "node_name_list" in data.keys():
+        node_name_list = data["node_name_list"]
+    else:
+        return None
 
     if "direction" in data.keys():
         direction = data["direction"]
@@ -1165,8 +1210,8 @@ def model_service_update_subgraph_by_multi_hop_multi_edge(data, config_params):
     if "edge_con_list_list" in data.keys():
         edge_con_list_list = data["edge_con_list_list"]
     else:
-        edge_con_list_list = [["downlink_length>10000", "protocol='http'"], ["record_date='2019-04-15'"]]
-
+        # edge_con_list_list = [["downlink_length>10000", "protocol='http'"], ["record_date='2019-04-15'"]]
+        edge_con_list_list = [[]]
     # client = Client('10.217.62.41')
     # graph = CHGraph(client)
 
@@ -1180,7 +1225,15 @@ def model_service_update_subgraph_by_multi_hop_multi_edge(data, config_params):
     #   )
 
     try:
-        status = graph.update_subgraph_by_multi_hop_multi_edge(
+        name2config = db.get_name2config(subgraph_name)
+        main_graph = name2config[0][5]
+        graph_schema = name2config[0][3]
+        schema = json.loads(graph_schema)
+        graph.use_graph(main_graph, db)
+
+        res = graph.update_subgraph_by_multi_hop_multi_edge(
+            main_graph,
+            db,
             subgraph_name,
             step,
             start_vertex_list,
@@ -1188,12 +1241,81 @@ def model_service_update_subgraph_by_multi_hop_multi_edge(data, config_params):
             edge_name_list,
             edge_con_list_list
         )
-        return status
+        graphEdges = []
+        for h in range(step):
+            # result["s_" + str(i)]["hop_" + str(h + 1)] = {}
+            for j in range(len(edge_name_list)):
+                res_tem = res[h][j].values.tolist()
+                if len(graphEdges) == len(edge_name_list):
+                    graph_edge = graphEdges[j]
+                    graph_edge["data"]["detail"] = graph_edge["data"]["detail"] + res_tem
+                else:
+                    graph_edge = {"data": {"detail": res_tem}}
+
+                graph_edge["type"] = edge_name_list[j]
+                schema_tem = res[h][j].columns.values.tolist()
+                graph_edge["data"]["schema"] = schema_tem
+                if len(graphEdges) < len(edge_name_list):
+                    graphEdges.append(graph_edge)
+        field_list = schema["vertexes"][node_name_list[0]]["fields"]
+        result = model_get_vertex_by_edge(config_params, main_graph, subgraph_name, graphEdges, node_name_list, field_list)
+        return result
     except Exception as e:
+        traceback.print_exc()
         print(e)
+        logger.error(e)
         return "update subgraph by multi hop multi edge failed"
 
 
+def model_get_vertex_by_edge(config_params, graph_name, subgraph_name, graphEdges, node_name_list, field_list=None):
+    node_tem = []
+    import numpy as np
+    for edge_one in range(len(graphEdges)):
+        if graphEdges[edge_one]["data"]["detail"]:
+            node_one = np.array(graphEdges[edge_one]["data"]["detail"])[:, 0:2].flatten().tolist()
+            node_tem += node_one
+    if node_tem:
+        node_data = {"subGraph": graph_name, "nodeTypes": node_name_list}
+        conditions = []
+        if node_name_list:
+            for l in range(len(node_name_list)):
+                condition = {"type": node_name_list[l], "id": node_tem}
+                conditions.append(condition)
+            node_data["nodeConditions"] = conditions
+        node_data["resultType"] = "def"
+        if field_list:
+            node_data["fieldList"] = field_list
+        model_service = mds.ModelService()
+        result = model_service.search_subgraph_by_condition(node_data, config_params)
+        graphNodes = result["pathList"]["graphNodes"]
+        new_graphNodes = convert_def_node(graphNodes)
+
+        result["pathList"]["graphNodes"] = new_graphNodes
+        result["pathList"]["graphEdges"] = graphEdges
+    else:
+        result = {"pathList": {"graphEdges": graphEdges, "graphNodes": node_tem}}
+
+    return result
+
+
+def insert_vertexes_data(subgraph_name, graphNodes):
+    pass
+
+
+def convert_def_node(graphNodes):
+    new_graphNodes = []
+    for graphNode in graphNodes:
+        defdata = graphNode["data"]
+        #schema = defdata.iloc[:, :2].columns.values.tolist()
+        #field_data = defdata.iloc[:, :2].values.tolist()
+        schema = defdata.iloc[:, :].columns.values.tolist()
+        field_data = defdata.iloc[:, :].values.tolist()
+        new_data = {"schema": schema, "detail": field_data}
+        new_graphNode = {"data": new_data, "type": graphNode["type"]}
+        new_graphNodes.append(new_graphNode)
+    return new_graphNodes
+
+
 def model_service_update_subgraph_by_match_edge(data, config_params):
     if "graph_name" in data.keys():
         graph_name = data["graph_name"]
@@ -1237,57 +1359,131 @@ def model_service_update_subgraph_by_find_path_multi_edge(data, config_params):
     if "graph_name" in data.keys():
         graph_name = data["graph_name"]
     else:
-        graph_name = "taobao"
+        # return None
+        pass
 
     if "subgraph_name" in data.keys():
         subgraph_name = data["subgraph_name"]
     else:
-        subgraph_name = "taobao_sub"
+        subgraph_name = graph_name + "_sub"
 
     # client = Client('10.217.62.41')
     #
     # graph = CHGraph(client)
-    graph = config_params["graph"]
+    # graph = config_params["graph"]
     db = config_params["db"]
-    graph.use_graph(graph_name, db)
+    clickhouse_connect = config_params["clickhouse_connect"]
+    graph_client = get_client(clickhouse_connect)
+    graph = CHGraph(graph_client)
 
     if "start_vertex_list" in data.keys():
         start_vertex_list = data["start_vertex_list"]
     else:
-        start_vertex_list = []
+        return None
 
     if "end_vertex_list" in data.keys():
         end_vertex_list = data["end_vertex_list"]
     else:
-        end_vertex_list = ['115.102.0.56']
+        return None
+
+    if "node_name_list" in data.keys():
+        node_name_list = data["node_name_list"]
+    else:
+        node_name_list = []
 
     if "edge_name_list" in data.keys():
         edge_name_list = data["edge_name_list"]
     else:
-        edge_name_list = ["service_date", "ip", "host", "speed"]
+        return None
 
     if "edge_con_list_list" in data.keys():
         edge_con_list_list = data["edge_con_list_list"]
     else:
-        edge_con_list_list = [["record_date='2017-5-13'"], ["record_date='2017-5-9'"]]
+        edge_con_list_list = [[]]
 
     if "step_limit" in data.keys():
         step_limit = data["step_limit"]
     else:
         step_limit = 1
 
+    # result = {}
     try:
-        graph.update_subgraph_by_find_path_multi_edge(subgraph_name,
-                                                      start_vertex_list,
-                                                      end_vertex_list,
-                                                      edge_name_list,
-                                                      edge_con_list_list,
-                                                      step_limit)
-        return "subgraph by find path multi edge success"
+        name2config = db.get_name2config(subgraph_name)
+        main_graph_name = name2config[0][5]
+        graph_schema = name2config[0][3]
+        schema = json.loads(graph_schema)
+        graph.use_graph(main_graph_name, db)
+
+        res = graph.update_subgraph_by_find_path_multi_edge(db,
+                                                            main_graph_name,
+                                                            subgraph_name,
+                                                            start_vertex_list,
+                                                            end_vertex_list,
+                                                            edge_name_list,
+                                                            edge_con_list_list,
+                                                            step_limit)
+        graphEdges = []
+        res_tem = []
+        schema_tem = []
+        # result["pathList"] = {}
+        if step_limit == 1:
+            for j in range(len(edge_name_list)):
+                graph_edge = {"data": {}}
+                res_tem = res[0][j].values.tolist()
+                schema_tem = res[0][j].columns.values.tolist()
+                graph_edge["type"] = edge_name_list[j]
+                graph_edge["data"]["schema"] = schema_tem
+                graph_edge["data"]["detail"] = res_tem
+                graphEdges.append(graph_edge)
+            # result["pathList"]["graphEdges"] = graphEdges
+            # result["hop_" + str(1)][edge_name_list[j]] = res[0][j].values.tolist()
+        elif step_limit >= 2:
+            for i in range(1, step_limit + 1):
+                # result["s_" + str(i)] = {}
+                if i == 1:
+                    # result["s_" + str(i)] = {}
+                    for j in range(len(edge_name_list)):
+                        graph_edge = {"data": {}}
+                        res_tem = res[i - 1][j].values.tolist()
+                        schema_tem = res[i - 1][j].columns.values.tolist()
+                        graph_edge["type"] = edge_name_list[j]
+                        graph_edge["data"]["schema"] = schema_tem
+                        graph_edge["data"]["detail"] = res_tem
+                        graphEdges.append(graph_edge)
+                        # result["s_" + str(i)][edge_name_list[j]] = res[i - 1][j].values.tolist()
+                else:
+                    # result["s_" + str(i)] = {}
+                    for h in range(step_limit):
+                        # result["s_" + str(i)]["hop_" + str(h + 1)] = {}
+                        for j in range(len(edge_name_list)):
+                            if h < len(res[i - 1]):
+                                graph_edge = graphEdges[j]
+                                res_tem = res[i - 1][h][j].values.tolist()
+                                schema_tem = res[i - 1][h][j].columns.values.tolist()
+                                graph_edge["type"] = edge_name_list[j]
+                                if schema_tem:
+                                    graph_edge["data"]["schema"] = schema_tem
+                                graph_edge["data"]["detail"] = graph_edge["data"]["detail"] + res_tem
+                                # graphEdges.append(graph_edge)
+                            else:
+                                continue
+                                # result["s_" + str(i)]["hop_" + str(h + 1)][edge_name_list[j]] = res[i - 1][h][
+                                #     j].values.tolist()
+                                # result["s_" + str(i)]["hop_" + str(h + 1)][edge_name_list[j]] = res[i - 1][h][
+                                #     j].values.tolist()
+                                # else:
+                                #     result["s_" + str(i)]["hop_" + str(h + 1)][edge_name_list[j]] = []
+                                #     result["s_" + str(i)]["hop_" + str(h + 1)][edge_name_list[j]] = []
+                                # mds.ModelService.search_subgraph_by_condition()
+        field_list = schema["vertexes"][node_name_list[0]]["fields"]
+        result = model_get_vertex_by_edge(config_params, main_graph_name, subgraph_name, graphEdges, node_name_list, field_list)
     except Exception as e:
         print(e)
+        print(e.args)
+        traceback.print_exc()
+        logger.error(e)
         return "subgraph by find path multi edge failed"
-    return
+    return result
 
 
 def model_service_destroy_subgraph(data, config_params):
